package com.sun.xml.internal.ws.runtime.config;

import javax.xml.ws.WebServiceFeature;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.XMLEvent;
import java.util.Iterator;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.XMLStreamException;
import com.sun.xml.internal.ws.config.metro.util.ParserUtil;
import javax.xml.ws.WebServiceException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.XMLEventReader;
import javax.xml.namespace.QName;
import com.sun.istack.internal.logging.Logger;
import com.sun.xml.internal.ws.config.metro.dev.FeatureReader;

public class TubelineFeatureReader implements FeatureReader
{
    private static final Logger LOGGER;
    private static final QName NAME_ATTRIBUTE_NAME;
    
    public TubelineFeatureReader() {
    }
    
    public TubelineFeature parse(XMLEventReader reader) throws WebServiceException {
        try {
            StartElement element = reader.nextEvent().asStartElement();
            boolean attributeEnabled = true;
            Iterator<Attribute> iterator = element.getAttributes();
            while (iterator.hasNext()) {
                Attribute nextAttribute = (Attribute)iterator.next();
                QName attributeName = nextAttribute.getName();
                if (TubelineFeatureReader.ENABLED_ATTRIBUTE_NAME.equals(attributeName)) {
                    attributeEnabled = ParserUtil.parseBooleanValue(nextAttribute.getValue());
                }
                else {
                    if (TubelineFeatureReader.NAME_ATTRIBUTE_NAME.equals(attributeName)) {
                        continue;
                    }
                    throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("Unexpected attribute"));
                }
            }
            return this.parseFactories(attributeEnabled, element, reader);
        }
        catch (XMLStreamException e) {
            throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("Failed to unmarshal XML document", e));
        }
    }
    
    private TubelineFeature parseFactories(boolean enabled, StartElement element, XMLEventReader reader) throws WebServiceException {
        int elementRead = 0;
    Label_0268:
        while (reader.hasNext()) {
            try {
                XMLEvent event = reader.nextEvent();
                switch (event.getEventType()) {
                    case 5: {
                        continue;
                    }
                    case 4: {
                        if (event.asCharacters().isWhiteSpace()) {
                            continue;
                        }
                        throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("No character data allowed, was " + event.asCharacters()));
                    }
                    case 1: {
                        ++elementRead;
                        continue;
                    }
                    case 2: {
                        if (--elementRead < 0) {
                            EndElement endElement = event.asEndElement();
                            if (!element.getName().equals(endElement.getName())) {
                                throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("End element does not match " + endElement));
                            }
                            break Label_0268;
                        }
                        else {
                            continue;
                        }
                    }
                    default: {
                        throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("Unexpected event, was " + event));
                    }
                }
            }
            catch (XMLStreamException e) {
                throw (WebServiceException)TubelineFeatureReader.LOGGER.logSevereException(new WebServiceException("Failed to unmarshal XML document", e));
            }
        }
        return new TubelineFeature(enabled);
    }
    
    static {
        LOGGER = Logger.getLogger(TubelineFeatureReader.class);
        NAME_ATTRIBUTE_NAME = new QName("name");
    }
}
